#pragma warning(disable:4786)
#include <stdio.h>
#include "OsiClpSolverInterface.hpp"
#include "OsiGlpkSolverInterface.hpp"
#include "OsiCpxSolverInterface.hpp"
#include "CoinPackedVector.hpp"

double precision = 1e-3;

bool checkEquality(double v1, double v2)
{
	if (!v2 && abs(v1) < precision)
		return true;

	return (abs(v1 / v2 - 1) < precision);
}

void check(bool b)
{
	if (!b) {
		printf("fail\n");
		//assert(false);
	}
}

void solveP1(OsiSolverInterface *s)
{
	double inf = s->getInfinity();

	CoinPackedMatrix m;

	m.transpose();

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	m.appendRow(r0);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	m.appendRow(r1);

	int numcol = 2;

	double *obj = new double[numcol];
	obj[0] = 3;
	obj[1] = 1;

	double *collb = new double[numcol];
	collb[0] = 0;
	collb[1] = 0;

	double *colub = new double[numcol];
	colub[0] = inf;
	colub[1] = inf;

	int numrow = 2;

	double *rowlb = new double[numrow];
	rowlb[0] = -inf;
	rowlb[1] = -inf;

	double *rowub = new double[numrow];
	rowub[0] = 10;
	rowub[1] = 15;

	s->loadProblem(m, collb, colub, obj, rowlb, rowub);

	delete [] obj;
	delete [] collb;
	delete [] colub;

	delete [] rowlb;
	delete [] rowub;

	s->setObjSense(-1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 5));
	check(checkEquality(s->getColSolution()[1], 0));

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 3));
	check(checkEquality(s->getColSolution()[1], 4));
}

void solveP2(OsiSolverInterface *s)
{
	double inf = s->getInfinity();

	CoinPackedMatrix m;

	m.transpose();

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	m.appendRow(r0);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	m.appendRow(r1);

	CoinPackedVector r2;
	r2.insert(0, 1);
	r2.insert(1, 1);
	m.appendRow(r2);

	int numcol = 2;

	double *obj = new double[numcol];
	obj[0] = 3;
	obj[1] = 1;

	double *collb = new double[numcol];
	collb[0] = 0;
	collb[1] = 0;

	double *colub = new double[numcol];
	colub[0] = inf;
	colub[1] = inf;

	int numrow = 3;

	double *rowlb = new double[numrow];
	rowlb[0] = -inf;
	rowlb[1] = -inf;
	rowlb[2] = 1;

	double *rowub = new double[numrow];
	rowub[0] = 10;
	rowub[1] = 15;
	rowub[2] = inf;

	s->loadProblem(m, collb, colub, obj, rowlb, rowub);

	delete [] obj;
	delete [] collb;
	delete [] colub;

	delete [] rowlb;
	delete [] rowub;

	s->setObjSense(-1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 5));
	check(checkEquality(s->getColSolution()[1], 0));

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 3));
	check(checkEquality(s->getColSolution()[1], 4));
}

void solveP3(OsiSolverInterface *s)
{
	double inf = s->getInfinity();

	CoinPackedVector empty;

	s->addCol(empty, 0, 10, 3);
	s->addCol(empty, 0, 10, 1);

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	s->addRow(r0, 0, 10);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	s->addRow(r1, 0, 15);

	s->setObjSense(-1);

	s->writeMps("test");

	s->resolve();
	bool optimal = s->isProvenOptimal();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 5));
	check(checkEquality(s->getColSolution()[1], 0));

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 3));
	check(checkEquality(s->getColSolution()[1], 4));
}

void solveP4(OsiSolverInterface *s)
{
	double inf = s->getInfinity();

	CoinPackedVector empty;

	s->addCol(empty, 0, inf, 3);
	s->addCol(empty, 0, inf, 1);

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	s->addRow(r0, -inf, 10);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	s->addRow(r1, -inf, 15);

	s->setObjSense(-1);

	s->writeMps("test");

	s->resolve();
	bool optimal = s->isProvenOptimal();

	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 5));
	check(checkEquality(s->getColSolution()[1], 0));

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 3));
	check(checkEquality(s->getColSolution()[1], 4));
}

void solveP5(OsiSolverInterface *s)
{
	double inf = s->getInfinity();

	CoinPackedVector empty;

	s->addCol(empty, 0, inf, 3);
	s->addCol(empty, 0, inf, 1);

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	s->addRow(r0, 0, 10);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	s->addRow(r1, 0, 15);

	s->setObjSense(-1);

	s->writeMps("test");

	s->resolve();
	bool optimal = s->isProvenOptimal();

	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 5));
	check(checkEquality(s->getColSolution()[1], 0));

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	printf("%f %f\n", s->getColSolution()[0], s->getColSolution()[1]);
	check(checkEquality(s->getColSolution()[0], 3));
	check(checkEquality(s->getColSolution()[1], 4));
}

void registerCplex()
{
	static char *ilm_CPLEX_license=\
	"LICENSE aisystems-bruxelles-Steel Planner Module 5 users\n\
	RUNTIME CPLEX      7.500 15-Feb-2003 7FCPU2G4SW3Z Y pcwinnt , options: m b , license: 436600 ";
	static int ilm_CPLEX_license_signature=1450604;

	if (!CPXRegisterLicense(ilm_CPLEX_license, ilm_CPLEX_license_signature)) {
		//assert(false); // It seems it complain but everything is actualy ok so.
	}
}

int main()
{
	OsiSolverInterface *s = NULL;

	s = new OsiClpSolverInterface();
	solveP1(s);
	delete s;
	s = new OsiClpSolverInterface();
	solveP2(s);
	delete s;
	s = new OsiClpSolverInterface();
	solveP3(s);
	delete s;
	s = new OsiClpSolverInterface();
	solveP4(s);
	delete s;
	s = new OsiClpSolverInterface();
	solveP5(s);
	delete s;

	s = new OsiGlpkSolverInterface();
	solveP1(s);
	delete s;
	s = new OsiGlpkSolverInterface();
	solveP2(s);
	delete s;
	s = new OsiGlpkSolverInterface();
	solveP3(s);
	delete s;
	s = new OsiGlpkSolverInterface();
	solveP4(s);
	delete s;
	s = new OsiGlpkSolverInterface();
	solveP5(s);
	delete s;

	registerCplex();
	s = new OsiCpxSolverInterface();
	solveP1(s);
	delete s;
	s = new OsiCpxSolverInterface();
	solveP2(s);
	delete s;
	s = new OsiCpxSolverInterface();
	solveP3(s);
	delete s;
	s = new OsiCpxSolverInterface();
	solveP4(s);
	delete s;
	s = new OsiCpxSolverInterface();
	solveP5(s);
	delete s;

	return 0;
}